/*************************************************************************
Giro-Tech presents the slap2000.
This code allows the user to record lap times on the watch by pressing the
start button. The left button is used to stop the recording.

It operates with three while loops. The first loop allows user to retore
old values from the EEPROM memory, see arduino documentation. This has a 
read/write cycle of 100,000 or so, but this should be sufficient. To do so
hold the left button.

The second loop allows the user to go over and save laps to memory. The 
right button goes through splits upwards, so 1, 2, 3, ... while the left 
button goes through the other way. If you hold both, you can save the laps.
Pressing the start button enters the third loop to record laps.

The third loop is used to actually record the laps. Pressing the start 
button records first lap. The lap should be displayed on the screen for
a bit then it will go back to displaying the live time of the lap.

I originally had the watch also record splits or the cummulative time, but
I had issues with it running out of memory on the ATmega328. This is done
by master[i] - master[0] which is cumulative time. 

*************************************************************************/


#include <Wire.h>
#include <EEPROM.h>
#include <Adafruit_GFX.h> //this lib must be added from adafruit
#include <Adafruit_SSD1306.h> //this one also


#define OLED_RESET 12 //Pin to set for the lcd screen
Adafruit_SSD1306 display(OLED_RESET);

#define LOGO16_GLCD_HEIGHT 16 
#define LOGO16_GLCD_WIDTH  16 

boolean running = false; //useful boolean for recording values
int runnum = 0; //value to keep track of lap number
unsigned long timeon = 0; //used for when the timer is turned off and then turned back on
unsigned long elapsedtime = 0; //is time to add to master (the timer that keeps track of time between laps)
int firstloop = 1; //to stop it from adding delay each time. 1 is yes

long previousMillis = 0; //used to display the time on the screen see example BlinkWithoutDelay
long interval = 100; //how often it updates clock counts 100 milliseconds
unsigned long laptime = 0; //records the lap time for display.

int displayvalue = 0; //for going over laps. is value in array (lap count minus 1)
boolean countup = false; //this used for going over splits


int storecounter = 0; //used to store values
boolean restore = true;//to restore values

unsigned long master[52]; //used to record the mills since reset of arduino. 51 values
unsigned long lap[51]; //used to display the value of the lap times. 50 laps


const int startbuttonpin = 3; //the digital input of start button, etc on the arduino
const int leftbuttonpin = 5; 
const int rightbuttonpin = 2;


void setup()   {                
  Serial.begin(9600);

  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)
  // init done

  display.clearDisplay();   // clears the screen and buffer  
  
  //Says Giro Tech. Note in order to print I found I needed to say the color and size
  display.setTextSize(2);
  display.setCursor(10,8);
  display.setTextColor(BLACK, WHITE); // 'inverted' text
  display.println("Giro Tech");
  display.setTextColor(WHITE);
  display.setTextSize(2);
  display.display();
  delay(2000);  //delay( x number of milliseconds)
  display.clearDisplay();
  display.display();  
  
  
//to get buttons to work
  pinMode(startbuttonpin, INPUT);  
  pinMode(leftbuttonpin, INPUT); 
  pinMode(rightbuttonpin, INPUT); 
  
  
}

void loop(){
  ///~~~~~~~~~~~~~~~~~~~~~~~~~~restore loop ~~~~~~~~~~~~~
  if (restore){
    display.setTextColor(WHITE);
    display.setTextSize(1);
    display.setCursor(0,0);
    display.println("Restore?");
    display.println("Left for yes");
    display.display();
    delay(1000);
    int leftbuttonState = digitalRead(leftbuttonpin);
    
    if (leftbuttonState == 1){
      display.clearDisplay();
      display.setTextColor(WHITE);
      display.setTextSize(1);
      display.setCursor(0,0);
      display.println("Restoring");
      display.display();
      restorevalue(); //function and next line restores the splits
      runnum = EEPROM.read(200); 
      for (int y = 0 ; y < runnum ; y++){
        master[y] = 444;
      } //give it an initial value so it works
      firstloop = 0; //to add on new splits
      display.clearDisplay();
      display.setTextColor(WHITE);
      display.setTextSize(2);
      display.setCursor(0,0);
      display.println("Restored");
      display.display();
      delay(900);
      restore = false;
      
    }
    display.clearDisplay();
    display.display();
    restore = false;
    
  }  
  //end of restore if 



//~~~~~~~~~~~~~~~~~~~~while not recording laps ~~~~~~~~~~~~~~~~~~~


  while (!running){
    int startbuttonState = digitalRead(startbuttonpin); //reading in the values
    delay(1);
    int leftbuttonState = digitalRead(leftbuttonpin); //delay is for stability
    delay(1);
    int rightbuttonState = digitalRead(rightbuttonpin);
    delay(1);
    
    //pressing the start button
    if (startbuttonState == 1){
     timeon=millis();
     onscreen("Starting..",300);
     running = true;
    }
        
   
    //holding left and right button at the same time 
    if ((leftbuttonState == 1) && (rightbuttonState == 1)){
      display.clearDisplay();
      display.setTextSize(2);
      display.setCursor(0,0);
      display.setTextColor(WHITE);
      display.println("Store? ");
      display.println(storecounter);
      display.display();
      delay(600);
      storecounter++;
      
      if (storecounter > 5){
        if (lap[0] != 0){
          display.clearDisplay();
          display.setTextSize(2);
          display.setCursor(0,0);
          display.setTextColor(WHITE);
          display.println("Storing ");
          display.display();
          storevalue(lap);
          onscreen("Stored.", 1000);
          storecounter = 0;
        }
        else {
          onscreen("No Laps.",900);
          storecounter = 0;
          
        }

      }

    }
    
  
    //Pressing the left button. This goes through laps counting down, ie 5, 4, 3, etc
    else if (leftbuttonState == 1){
      if (lap[0] == 0){
        onscreen("No Laps",500);        
      }
      
      
      else{
        if (countup){
          displayvalue = (displayvalue + (runnum - 3) ) % (runnum - 1 ); 
          countup = false; //runnum minus 1 because in loop it is one higher than number of laps
        }
        display.clearDisplay();
        display.setTextSize(2);
        display.setCursor(0,0);
        display.setTextColor(WHITE);
        display.println("Lap " + String( (displayvalue + 1),DEC) );
        display.println( String (timestring(lap[displayvalue]) )  );        
        display.display();
        displayvalue = ( (displayvalue + (runnum - 2 ) ) % (runnum-1) );
        delay(100);
        
          
      }  
      
    }
    
    //Pressing the right button. go through laps going up, ie 1, 2, 3, ...
    else if (rightbuttonState == 1){
      if (lap[0] == 0){
        onscreen("No Laps",500);  
        display.display();     
      }
      
      else{
        if (!countup){
          displayvalue = ( displayvalue + 2 ) % (runnum - 1);
          countup = true;
        }
        display.clearDisplay();
        display.setTextSize(2);
        display.setCursor(0,0);
        display.setTextColor(WHITE);
        display.println("Lap " + String( (displayvalue + 1),DEC) );
        display.println( String (timestring(lap[displayvalue]) )  );
        display.display();
        displayvalue = ( (displayvalue + 1 ) % (runnum-1) );
        delay(100);

        
        
          
      }
      
    }
    
  }
  

// ~~~~~~~~~~~~~~~~~~~~~~~~~while recording laps ~~~~~~~~~~~~~~~~~~~~~~~~~
  //Normal operation will be the time scrolling after the first slap.
  while (running){    
    if (runnum == 0){
      masterfun();
      delay(100);
      runnum = 1;
    }
    
    if (runnum == 50){
      onscreen("Error", 400);
      break;
    }
    
    //recording lap
    int startbuttonState = digitalRead(startbuttonpin); //reading in the values. if high, State = 1
    delay(1);
    if (startbuttonState == 1){
      if (firstloop == 0){
        elapsedtime = millis() - timeon;
      }
      slap();
      timeon = millis();
      display.clearDisplay();
      display.setTextSize(2);
      display.setCursor(0,0);
      display.setTextColor(WHITE);
      display.println( String( timestring(lap[runnum-1]) )  );     
      display.display();
      runnum = runnum + 1;
      delay(700);
          
    }
    
    
    
    unsigned long currentMillis = millis(); 
    laptime = millis() - timeon;
    
    if( currentMillis - previousMillis > interval){  //this if loop updates time on clock.
      previousMillis = currentMillis;
      display.clearDisplay();
      display.setTextSize(2);
      display.setCursor(0,0);
      display.setTextColor(WHITE);
      display.println(String(timestring(laptime)) );
      display.display();
      
      
    }
    
    
      
    int leftbuttonState = digitalRead(leftbuttonpin); //delay is for stability
    delay(1);  
    if (leftbuttonState == 1){
      running = false; //so it goes to while loop around
      firstloop = 0;
      displayvalue = 0;
      onscreen("Stop",500);
      break;  
      
    }  
    
    
    
    
    
  } //End of recording laps loop


  
  
  
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~Functions~~~~~~~~~~~~~~~~~~~~~~~

//displays values on the lcd screen
void onscreen(String bob, int delayvalue) {
  display.clearDisplay();
  display.setTextSize(2);
  display.setCursor(0,0);
  display.setTextColor(WHITE);
  display.println(bob);
  display.display();
  delay(delayvalue);
  display.clearDisplay();
  display.display();
  
}


// function records the  master time.
unsigned long* masterfun(){
  for(int i = 0; i < 101; i++){
    if(master[i] == 0){
      if (runnum == 0){
        master[i] = millis();
        return master;
        break;
      }
      if (firstloop == 1){
        master[i] = millis();
        return master;
        break;
      }

      if(firstloop == 0){
        master[i] = ( elapsedtime + master[i-1] );
        return master;
        break;
      }
   
    }  
   
  }
  
}

//function that converts the master times to the lap times 
unsigned long* lapfun(){
  for(int i = 2; i < 101; i++){
    if(master[1] == 0){
      break;
    }
    if(master[i] == 0){    //this step find the last lap that needs to be updated
      lap[i-2] = (master[i-1] - master[i-2]);
      return lap;
      break;

    }  
   
  }
  
}


//to use the two functions masterfun and lapfun. 
//might work better with some kind of interrupt.
void slap(){
  masterfun();
  lapfun();
  delay(90); // so button isn't accidently pressed twice.
  
  
}



//function that converts the millisecond values to a clear display for laps
//so it converts 1000 to 0:01.00
char* timestring(unsigned long x){
  int fracmilli = (x % 1000);
  int frachund = fracmilli / 10; //for just hundreths of a second
  int sec = x / 1000;
  int fracsec = (sec % 60);
  int minu = sec / 60;
  String secStr = "ERR";
  String hundStr = "ERR";
  String zero = "0";
  // next few lines is to add in the nice looking zeros.
  if (fracsec < 10){
    String fracsecStr = String(fracsec,DEC);
    secStr =  zero + fracsecStr;
  }
  if (fracsec > 9){
    secStr = String(fracsec,DEC);
  }
  if (frachund < 10){
    String frachundStr = String(frachund,DEC);
    hundStr =  zero + frachundStr;
  }
  if (frachund > 9){
    hundStr = String(frachund,DEC);
  }
  String stringmin = String(minu,DEC);
  String colon = ":";
  String period = ".";
  String stringout = stringmin + colon + secStr + period + hundStr;
  char output[stringout.length()+1]; 
  stringout.toCharArray(output,stringout.length()+1);
  return output;
  
  
  
}

//function that stores the laps in EEPROM see arduino documentation
//if not saved this way they will be deleted when power goes out
//note that this is written byte by byte and is limited to 4.5 hours
void storevalue(unsigned long* laparray){
  int bytevalue = 0;
  
  for (int l = 0; l<=50; l++){
    unsigned long x = laparray[l];
    int fracmilli = (x % 1000);
    int frachund = fracmilli / 10; //for just hundreths of a second
    int sec = x / 1000;
    int fracsec = (sec % 60);
    int minu = sec / 60;
    EEPROM.write(bytevalue, minu);
    EEPROM.write(bytevalue+1, fracsec);
    EEPROM.write(bytevalue+2, frachund);
    bytevalue = bytevalue + 3;
  }
  EEPROM.write(200, runnum); //need the runnum to scroll through.
  
  
}


//function that restores the laps
//saved to EEPROM
unsigned long* restorevalue(){
  int bytevalue = 0;
  int lapnumber = 0;
  
  
  while (bytevalue < 150 ){
    int minut = EEPROM.read(bytevalue);
    int secon = EEPROM.read(bytevalue+1);
    int hund =  EEPROM.read(bytevalue+2);
    unsigned long newlap = (60000.0 * minut) + (1000.0 * secon) + (10.0 * hund);
    lap[lapnumber] = newlap;
    lapnumber = lapnumber + 1;
    bytevalue = bytevalue + 3;

  }
  
  return lap;
  
  
}


